######################################################################
#  Copyright (c) 2008-2013, Alliance for Sustainable Energy.
#  All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
######################################################################

module EnergyPlus

  require 'pathname'

  class StatFile
    attr_accessor :path
    attr_accessor :valid
    attr_accessor :lat
    attr_accessor :lon
    attr_accessor :elevation
    attr_accessor :gmt
    attr_accessor :monthlyDB
    attr_accessor :hdd18
    attr_accessor :cdd18
    attr_accessor :hdd10
    attr_accessor :cdd10

    def initialize(path)
      @path = Pathname.new(path)
      @valid = false
      @lat = []
      @lon = []
      @gmt = []
      @elevation = []
      @hdd18 = []
      @cdd18 = []
      @hdd10 = []
      @cdd10 = []
      @monthlyDB = []
      @deltaDB = []
      init
    end

    def valid?
      return @valid
    end

    # the mean of the mean monthly dry bulbs
    def meanDB
      if not @monthlyDB.empty? then
        thisSum = 0
        @monthlyDB.each { |db| thisSum += db }
        thisMean = thisSum/@monthlyDB.size
      else
        thisMean = ""
      end
      return thisMean
    end

    # max - min of the mean monthly dry bulbs
    def deltaDB
      if not @monthlyDB.empty? then
        deltaT = @monthlyDB.max-@monthlyDB.min
      else
        deltaT = ""
      end

      return deltaT
    end

    private

    # initialize
    def init
      if @path.exist?
        File.open(@path) do |f|
          text = f.read
          parse(text)
        end
      end
    end

    def parse(text)

      # get lat, lon, gmt
      regex = /\{(N|S)\s*([0-9]*).\s*([0-9]*)'\}\s*\{(E|W)\s*([0-9]*).\s*([0-9]*)'\}\s*\{GMT\s*(.*)\s*Hours\}/
      matchData = text.match(regex)
      if matchData.nil?
        puts "Can't find lat/lon/gmt"
        return
      else

        @lat = matchData[2].to_f + (matchData[3].to_f)/60.0
        if matchData[1] == 'S'
          @lat = -@lat
        end

        @lon = matchData[5].to_f + (matchData[6].to_f)/60.0
        if matchData[4] == 'W'
          @lon = -@lon
        end

        @gmt = matchData[7]

      end

      # get elevation
      regex = /Elevation --\s*(.*)m (above|below) sea level/
      matchData = text.match(regex)
      if matchData.nil?
        puts "Can't find elevation"
        return
      else
        @elevation = matchData[1].to_f
        if matchData[2] == 'below'
          @elevation = -@elevation
        end
      end


      # get heating and cooling degree days
      cdd10Regex = /-\s*(.*) annual \(standard\) cooling degree-days \(10.C baseline\)/
      matchData = text.match(cdd10Regex)
      if matchData.nil?
        puts "Can't find CDD 10"
        return
      else
        @cdd10 = matchData[1].to_f
      end

      hdd10Regex = /-\s*(.*) annual \(standard\) heating degree-days \(10.C baseline\)/
      matchData = text.match(hdd10Regex)
      if matchData.nil?
        puts "Can't find HDD 10"
        return
      else
        @hdd10 = matchData[1].to_f
      end

      cdd18Regex = /-\s*(.*) annual \(standard\) cooling degree-days \(18.3.C baseline\)/
      matchData = text.match(cdd18Regex)
      if matchData.nil?
        puts "Can't find CDD 18"
        return
      else
        @cdd18 = matchData[1].to_f
      end

      hdd18Regex = /-\s*(.*) annual \(standard\) heating degree-days \(18.3.C baseline\)/
      matchData = text.match(hdd18Regex)
      if matchData.nil?
        puts "Can't find HDD 18"
        return
      else
        @hdd18 = matchData[1].to_f
      end


      #use regex to get the temperatures
      regex = /Daily Avg(.*)\n/
      matchData = text.match(regex)
      if matchData.nil?
        puts "Can't find outdoor air temps"
        return
      else
        # first match is outdoor air temps
        monthlyTemps = matchData[1].strip.split(/\s+/)

        # have to be 12 months
        if monthlyTemps.size != 12
          puts "Can't find outdoor air temps"
          return
        end

        # insert as numbers
        monthlyTemps.each {|temp| @monthlyDB << temp.to_f}
      end

      # now we are valid
      @valid = true
    end

  end
end